package com.flutter.braintree_flutter

import android.app.Activity
import android.content.Context
import android.content.Intent
import com.braintreepayments.api.dropin.DropInActivity
import com.braintreepayments.api.dropin.DropInRequest
import com.braintreepayments.api.dropin.DropInResult
import com.braintreepayments.api.models.GooglePaymentRequest
import com.braintreepayments.api.models.PayPalRequest
import com.google.android.gms.wallet.TransactionInfo
import com.google.android.gms.wallet.WalletConstants
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.PluginRegistry
import java.util.HashMap


class BraintreeFlutterPlugin : MethodChannel.MethodCallHandler, PluginRegistry.ActivityResultListener {
    private val DROP_IN_REQUEST_CODE = 0x1337

    private var activity: Activity?
    private var context: Context?
    private var activeResult: MethodChannel.Result? = null

    constructor (registrar: PluginRegistry.Registrar) {
        activity = registrar.activity()
        context = registrar.context()
        registrar.addActivityResultListener(this)
    }

    companion object {
        private val CHANNEL = "braintree_flutter"
        @JvmStatic
        fun registerWith(registrar: PluginRegistry.Registrar) {
            val channel = MethodChannel(registrar.messenger(), CHANNEL)
            channel.setMethodCallHandler(BraintreeFlutterPlugin(registrar))
        }
    }

    override fun onMethodCall(call: MethodCall?, result: MethodChannel.Result?) {
        if (call?.method?.equals("startDropIn")!!) {
            val clientToken = call.argument<String>("clientToken")
            val tokenizationKey = call.argument<String>("tokenizationKey")
            val dropInRequest = DropInRequest()
                    .amount(call.argument<String>("amount"))
                    .collectDeviceData(call.argument<Boolean>("collectDeviceData")!!)
                    .requestThreeDSecureVerification(call.argument<Boolean>("requestThreeDSecureVerification")!!)
                    .maskCardNumber(call.argument<Boolean>("maskCardNumber")!!)
                    .vaultManager(call.argument<Boolean>("vaultManagerEnabled")!!)
                    .disablePayPal()
                    .disableCard()

            if (clientToken != null)
                dropInRequest.clientToken(clientToken)
            else if (tokenizationKey != null)
                dropInRequest.tokenizationKey(tokenizationKey)
            if (call.argument<Boolean>("googlePaymentEnabled")!!)
                readGooglePaymentParameters(dropInRequest, call)
            if (!(call.argument<Boolean>("venmoEnabled")!!))
                dropInRequest.disableVenmo()
            // if (!((Boolean) call.argument("cardEnabled")))
            //   dropInRequest.disableCard();

            activeResult = result
            activity?.startActivityForResult(dropInRequest.getIntent(context), DROP_IN_REQUEST_CODE)
        } else {
            result?.notImplemented()
        }
    }

    private fun readGooglePaymentParameters(dropInRequest: DropInRequest, call: MethodCall) {
        if (!(call.argument<Boolean>("googlePaymentEnabled"))!!)
            return
        val googlePaymentRequest = GooglePaymentRequest()
                .transactionInfo(TransactionInfo.newBuilder()
                        .setTotalPrice((call.argument<String>("googlePaymentRequest_totalPrice"))!!)
                        .setCurrencyCode((call.argument<String>("googlePaymentRequest_currencyCode"))!!)
                        .setTotalPriceStatus(WalletConstants.TOTAL_PRICE_STATUS_FINAL)
                        .build())
                .billingAddressRequired((call.argument<Boolean>("googlePaymentRequest_billingAddressRequired"))!!)
//                .googleMerchantId("p53gk4q6g22mfjhm")

        dropInRequest.googlePaymentRequest(googlePaymentRequest)
    }


    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        when (requestCode) {
            DROP_IN_REQUEST_CODE -> {
                if (resultCode == Activity.RESULT_OK) {
                    val dropInResult = data?.getParcelableExtra<DropInResult>(DropInResult.EXTRA_DROP_IN_RESULT)
                    val paymentMethodNonce = dropInResult?.paymentMethodNonce
                    val map = HashMap<String, Any>()
                    map["token"] = paymentMethodNonce!!.nonce
//                    map["paymentMethodNonce_typeLabel"] = paymentMethodNonce.typeLabel
//                    map["paymentMethodNonce_description"] = paymentMethodNonce.description
//                    map["paymentMethodNonce_isDefault"] = paymentMethodNonce.isDefault
//                    map["deviceData"] = dropInResult.deviceData!!
                    activeResult?.success(map)
                } else if (resultCode == Activity.RESULT_CANCELED) {
                    activeResult?.success(null)
                } else {
                    val error = data?.getSerializableExtra(DropInActivity.EXTRA_ERROR) as Exception
                    activeResult?.error("DROP_IN_ERROR", error.message, null)
                }
                return true
            }
            else -> return false
        }
    }
}
