import 'package:flutter/material.dart';

import 'package:braintree_flutter/model/google_payment.dart';
import 'package:braintree_flutter/model/drop_in_request.dart';
import 'package:braintree_flutter/model/drop_in_result.dart';
import 'dart:io' show Platform;
import 'package:braintree_flutter/model/apple_payment.dart';
import 'package:braintree_flutter/model/single_payment_item.dart';

void main() => runApp(MyApp());

class MyApp extends StatefulWidget {
  MyApp({Key key, this.title}) : super(key: key);
  String title;

  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
//  String _platformVersion = 'Unknown';
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
        home: Scaffold(
      key: _scaffoldKey,
      appBar: AppBar(
        // Here we take the value from the MyHomePage object that was created by
        // the App.build method, and use it to set our appbar title.
        title: Text("Braintree Flutter Plugin"),
      ),
      body: Center(
        // Center is a layout widget. It takes a single child and positions it
        // in the middle of the parent.
        child: RaisedButton(
          onPressed: () {
            makePaymentRequest(context);
          },
          child: Text("Select Payment Method"),
          padding: EdgeInsets.all(16.0),
        ),
      ),
      // This trailing comma makes auto-formatting nicer for build methods.
    ));
  }

  void makePaymentRequest(BuildContext context) async {
    // for Apple pay, make payment items one by one using single payment object.
    List<SinglePaymentItem> items = <SinglePaymentItem>[];
    SinglePaymentItem firstItem =
        SinglePaymentItem(label: "Shirt", price: 1.00);
    SinglePaymentItem secondItem =
        SinglePaymentItem(label: "Pant", price: 2.00);
    SinglePaymentItem thirdItem =
        SinglePaymentItem(label: "Shoes", price: 4.00);
    SinglePaymentItem forthItem =
        SinglePaymentItem(label: "Shipping", price: 5.00);
    SinglePaymentItem fifthItem =
        SinglePaymentItem(label: "Total", price: 12.00);
    items.add(firstItem);
    items.add(secondItem);
    items.add(thirdItem);
    items.add(forthItem);
    items.add(fifthItem);

    var request = BraintreeDropInRequest(
        googlePaymentDropIn: GooglePaymentDropIn(
            currencyCode: 'USD',
            billingAddressRequired: true,
            totalPrice: '0.01'),
        applePaymentDropIn: ApplePaymentDropIn(
            totalPrice: "9.00",
            currencyCode: "CAD",
            countryCode: "CA",
            billingAddressRequired: false,
            items: items),
        collectDeviceData: true,
        vaultManagerEnabled: true,
        amount: '1.00',
        clientToken: "your-valid-client-token");

    String result = await request.start(request);
    if (result != null) {
      print(result);
      _showSnackbar(context, Colors.green, "Payment succeeded");
    } else {
      _showSnackbar(context, Colors.red, "Payment declined");
    }
  }

  void _showSnackbar(BuildContext context, Color color, String message) {
    final snackBar = new SnackBar(
      content: new Text(message),
      backgroundColor: color,
      duration: Duration(seconds: 5),
    );

    // Find the Scaffold in the Widget tree and use it to show a SnackBar!

    _scaffoldKey.currentState.showSnackBar(snackBar);
  }

  void getToken() {}
}
