import Flutter
import UIKit
import BraintreeDropIn
import Braintree

public class SwiftBraintreeFlutterPlugin: NSObject, FlutterPlugin {
    var braintreeClient: BTAPIClient?
    var flutterResult: FlutterResult?
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "braintree_flutter", binaryMessenger: registrar.messenger())
        let instance = SwiftBraintreeFlutterPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ methodCall: FlutterMethodCall, result: @escaping FlutterResult) {
        if (methodCall.method == "startDropIn") {
            let elementsArray = methodCall.arguments as! NSArray
            let elements = elementsArray[0] as! NSDictionary
            
            if elements.object(forKey: "tokenizationKey") as? String != nil{
                let token = elements.value(forKey: "tokenizationKey") as! String
            }
            flutterResult = result
            if elements.object(forKey: "clientToken") as? String != nil {
                let clientToken = elements.value(forKey: "clientToken") as! String
                self.braintreeClient = BTAPIClient(authorization: clientToken)
                self.showDropIn(clientTokenOrTokenizationKey: clientToken, completion: {payment, error in
                    if error != nil{
                        print(error!)
                    }else{
                        print(payment)
                    }
                }, methodCall)
                
            }else{
                result(FlutterMethodNotImplemented)
            }
        }
    }
    
    func showDropIn(clientTokenOrTokenizationKey: String, completion: @escaping (PKPayment?, Error?) -> Void, _ methodCall: FlutterMethodCall) {
        self.braintreeClient = BTAPIClient(authorization: clientTokenOrTokenizationKey)
        let elementsArray = methodCall.arguments as! NSArray
        let elements = elementsArray[0] as! NSDictionary
        let request =  BTDropInRequest()
        request.paypalDisabled = true
        let canMakePayments = PKPaymentAuthorizationViewController.canMakePayments() && PKPaymentAuthorizationViewController.canMakePayments(usingNetworks: [.visa, .masterCard])
        request.applePayDisabled = !canMakePayments
        request.cardDisabled = true
        request.amount = elements.value(forKey: "amount") as! String
        request.vaultManager = elements.value(forKey: "vaultManagerEnabled") as! Bool
        
        
        let dropIn = BTDropInController(authorization: clientTokenOrTokenizationKey, request: request)
        { (controller, result, error) in
            if (error != nil) {
                print("ERROR")
            } else if (result?.isCancelled == true) {
                print("CANCELLED")
            } else if let result = result {
                print(result)
                
                switch result.paymentOptionType {
                case .applePay, .masterCard :
                    if result.paymentMethod != nil{
                        //paymentMethod.nonce  You can use  nonce now
                        controller.dismiss(animated: true, completion: nil)
                    }else{
                        controller.dismiss(animated: true, completion: {
                            self.createPaymentRequest(methodCall)
                        })
                        
                    }
                default:
                    controller.dismiss(animated: true, completion: nil)
                }
            }
            controller.dismiss(animated: true, completion: nil)
        }
        
        UIApplication.shared.delegate?.window??.rootViewController?.present(dropIn!, animated: true, completion: nil)
    }
    
    private func createPaymentRequest(_ methodCall: FlutterMethodCall){
        let paymentNetworks:[PKPaymentNetwork] = [.amex,.masterCard,.visa]
        
        if PKPaymentAuthorizationViewController.canMakePayments(usingNetworks: paymentNetworks) {
            let elementsArray = methodCall.arguments as! NSArray
            let elements = elementsArray[0] as! NSDictionary
            let request = PKPaymentRequest()
            
            request.merchantIdentifier = "merchant.com.sacreserv.sacreservapp"
            request.countryCode = elements.value(forKey: "applePaymentRequest_countryCode") as! String
            request.currencyCode = elements.value(forKey: "applePaymentRequest_currencyCode") as! String
            request.supportedNetworks = paymentNetworks
            
            request.merchantCapabilities = .capability3DS
            
            if elements.value(forKey: "applePaymentRequest_billingAddressRequired") as! Bool {
                request.requiredBillingAddressFields = .postalAddress
            }
            let itemsArray = elementsArray[1] as! NSArray
            var paymentItems: [PKPaymentSummaryItem] = []
            for item in itemsArray{
                let itemDictionary = item as! NSDictionary
                let itemLabel = itemDictionary.value(forKey: "label") as! String
                let itemAmount = itemDictionary.value(forKey: "price") as! NSNumber
                let paymentItem = PKPaymentSummaryItem(label: itemLabel, amount: NSDecimalNumber(decimal: itemAmount.decimalValue))
                paymentItems.append(paymentItem)
            }
            
            //            let tshirt = PKPaymentSummaryItem(label: "T-shirt", amount: NSDecimalNumber(decimal:1.00))
            //            let shipping = PKPaymentSummaryItem(label: "Shipping", amount: NSDecimalNumber(decimal:1.00))
            //            let tax = PKPaymentSummaryItem(label: "Tax", amount: NSDecimalNumber(decimal:1.00))
            
            //            let itemDictionary = elements.value(forKey: "applePaymentRequest_Item") as! NSDictionary
            //            let itemLabel = itemDictionary.value(forKey: "label") as! String
            //            let itemAmount = itemDictionary.value(forKey: "price") as! NSNumber
            
            
            //            let item = PKPaymentSummaryItem(label: "Shirt", amount: NSDecimalNumber(decimal: 5.00))
            //            let total = PKPaymentSummaryItem(label: "Total", amount: NSDecimalNumber(decimal:3.00))
            request.paymentSummaryItems = paymentItems
            
            if let vc = PKPaymentAuthorizationViewController(paymentRequest: request)
                as PKPaymentAuthorizationViewController?
            {
                vc.delegate = self as? PKPaymentAuthorizationViewControllerDelegate
                UIApplication.shared.delegate?.window??.rootViewController?.present(vc, animated: true, completion: nil)
            } else {
                print("Error: Payment request is invalid.")
            }
        }
    }
}

extension Decimal {
    var doubleValue:Double {
        return NSDecimalNumber(decimal:self).doubleValue
    }
}

extension SwiftBraintreeFlutterPlugin: PKPaymentAuthorizationViewControllerDelegate{
    
    public func paymentAuthorizationViewController(_ controller: PKPaymentAuthorizationViewController, didAuthorizePayment payment: PKPayment, completion: @escaping ((PKPaymentAuthorizationStatus) -> Void)) {
        let applePayClient = BTApplePayClient(apiClient: braintreeClient!)
        applePayClient.tokenizeApplePay(payment) {
            (tokenizedApplePayPayment, error) in
            guard let tokenizedApplePayPayment = tokenizedApplePayPayment else {
                // Tokenization failed. Check `error` for the cause of the failure.
                
                // Indicate failure via completion callback.
                completion(PKPaymentAuthorizationStatus.failure)
                if self.flutterResult != nil {
                    self.flutterResult!("cannot get token")
                }
                return
            }
            
            // Received a tokenized Apple Pay payment from Braintree.
            
            // Send the nonce to your server for processing.
            print("nonce = \(tokenizedApplePayPayment.nonce)")
            completion(PKPaymentAuthorizationStatus.success)
            if self.flutterResult != nil {
                self.flutterResult!(["token":tokenizedApplePayPayment.nonce])
            }
        }
        
    }
    
    public func paymentAuthorizationViewControllerDidFinish(_ controller: PKPaymentAuthorizationViewController) {
        controller.dismiss(animated: true, completion: nil)
    }
}
